@@ -1,32 +1,9 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 // This header is private to the ProtobolBuffers library and must NOT be
 // included by any sources outside this library. The contents of this file are
@@ -37,12 +14,12 @@
 
 // Describes attributes of the field.
 typedef NS_OPTIONS(uint16_t, GPBFieldFlags) {
-  GPBFieldNone            = 0,
+  GPBFieldNone = 0,
   // These map to standard protobuf concepts.
-  GPBFieldRequired        = 1 << 0,
-  GPBFieldRepeated        = 1 << 1,
-  GPBFieldPacked          = 1 << 2,
-  GPBFieldOptional        = 1 << 3,
+  GPBFieldRequired = 1 << 0,
+  GPBFieldRepeated = 1 << 1,
+  GPBFieldPacked = 1 << 2,
+  GPBFieldOptional = 1 << 3,
   GPBFieldHasDefaultValue = 1 << 4,
 
   // Indicate that the field should "clear" when set to zero value. This is the
@@ -52,7 +29,7 @@ typedef NS_OPTIONS(uint16_t, GPBFieldFlags) {
   // Indicates the field needs custom handling for the TextFormat name, if not
   // set, the name can be derived from the ObjC name.
   GPBFieldTextFormatNameCustom = 1 << 6,
-  // Indicates the field has an enum descriptor.
+  // This flag has never had any meaning, it was set on all enum fields.
   GPBFieldHasEnumDescriptor = 1 << 7,
 
   // These are not standard protobuf concepts, they are specific to the
@@ -60,25 +37,46 @@ typedef NS_OPTIONS(uint16_t, GPBFieldFlags) {
 
   // These bits are used to mark the field as a map and what the key
   // type is.
-  GPBFieldMapKeyMask     = 0xF << 8,
-  GPBFieldMapKeyInt32    =  1 << 8,
-  GPBFieldMapKeyInt64    =  2 << 8,
-  GPBFieldMapKeyUInt32   =  3 << 8,
-  GPBFieldMapKeyUInt64   =  4 << 8,
-  GPBFieldMapKeySInt32   =  5 << 8,
-  GPBFieldMapKeySInt64   =  6 << 8,
-  GPBFieldMapKeyFixed32  =  7 << 8,
-  GPBFieldMapKeyFixed64  =  8 << 8,
-  GPBFieldMapKeySFixed32 =  9 << 8,
+  GPBFieldMapKeyMask = 0xF << 8,
+  GPBFieldMapKeyInt32 = 1 << 8,
+  GPBFieldMapKeyInt64 = 2 << 8,
+  GPBFieldMapKeyUInt32 = 3 << 8,
+  GPBFieldMapKeyUInt64 = 4 << 8,
+  GPBFieldMapKeySInt32 = 5 << 8,
+  GPBFieldMapKeySInt64 = 6 << 8,
+  GPBFieldMapKeyFixed32 = 7 << 8,
+  GPBFieldMapKeyFixed64 = 8 << 8,
+  GPBFieldMapKeySFixed32 = 9 << 8,
   GPBFieldMapKeySFixed64 = 10 << 8,
-  GPBFieldMapKeyBool     = 11 << 8,
-  GPBFieldMapKeyString   = 12 << 8,
+  GPBFieldMapKeyBool = 11 << 8,
+  GPBFieldMapKeyString = 12 << 8,
+
+  // If the enum for this field is "closed", meaning that it:
+  // - Has a fixed set of named values.
+  // - Encountering values not in this set causes them to be treated as unknown
+  //   fields.
+  // - The first value (i.e., the default) may be nonzero.
+  // NOTE: This could be tracked just on the GPBEnumDescriptor, but to support
+  // previously generated code, there would be not data to get the behavior
+  // correct, so instead it is tracked on the field. If old source compatibility
+  // is removed, this could be removed and the GPBEnumDescription fetched from
+  // the GPBFieldDescriptor instead.
+  GPBFieldClosedEnum = 1 << 12,
 };
 
 // NOTE: The structures defined here have their members ordered to minimize
 // their size. This directly impacts the size of apps since these exist per
 // field/extension.
 
+typedef struct GPBFileDescription {
+  // The proto package for the file.
+  const char *package;
+  // The objc_class_prefix option if present.
+  const char *prefix;
+  // The file's proto syntax.
+  GPBFileSyntax syntax;
+} GPBFileDescription;
+
 // Describes a single field in a protobuf as it is represented as an ivar.
 typedef struct GPBMessageFieldDescription {
   // Name of ivar.
@@ -88,11 +86,9 @@ typedef struct GPBMessageFieldDescription {
     // kept around right now for backwards compatibility.
     // clazz is used iff GPBDescriptorInitializationFlag_UsesClassRefs is set.
     char *className;  // Name of the class of the message.
-    Class clazz;  // Class of the message.
-    // For enums only: If EnumDescriptors are compiled in, it will be that,
-    // otherwise it will be the verifier.
+    Class clazz;      // Class of the message.
+    // For enums only.
     GPBEnumDescriptorFunc enumDescFunc;
-    GPBEnumValidationFunc enumVerifier;
   } dataTypeSpecific;
   // The field number for the ivar.
   uint32_t number;
@@ -109,8 +105,8 @@ typedef struct GPBMessageFieldDescription {
   GPBDataType dataType;
 } GPBMessageFieldDescription;
 
-// Fields in messages defined in a 'proto2' syntax file can provide a default
-// value. This struct provides the default along with the field info.
+// If a message uses fields where they provide default values that are non zero, then this
+// struct is used to provide the values along with the field info.
 typedef struct GPBMessageFieldDescriptionWithDefault {
   // Default value for the ivar.
   GPBGenericValue defaultValue;
@@ -120,10 +116,10 @@ typedef struct GPBMessageFieldDescriptionWithDefault {
 
 // Describes attributes of the extension.
 typedef NS_OPTIONS(uint8_t, GPBExtensionOptions) {
-  GPBExtensionNone          = 0,
+  GPBExtensionNone = 0,
   // These map to standard protobuf concepts.
-  GPBExtensionRepeated      = 1 << 0,
-  GPBExtensionPacked        = 1 << 1,
+  GPBExtensionRepeated = 1 << 0,
+  GPBExtensionPacked = 1 << 1,
   GPBExtensionSetWireFormat = 1 << 2,
 };
 
@@ -132,10 +128,10 @@ typedef struct GPBExtensionDescription {
   GPBGenericValue defaultValue;
   const char *singletonName;
   // Before 3.12, `extendedClass` was just a `const char *`. Thanks to nested
-  // initialization (https://en.cppreference.com/w/c/language/struct_initialization#Nested_initialization)
-  // old generated code with `.extendedClass = GPBStringifySymbol(Something)`
-  // still works; and the current generator can use `extendedClass.clazz`, to
-  // pass a Class reference.
+  // initialization
+  // (https://en.cppreference.com/w/c/language/struct_initialization#Nested_initialization) old
+  // generated code with `.extendedClass = GPBStringifySymbol(Something)` still works; and the
+  // current generator can use `extendedClass.clazz`, to pass a Class reference.
   union {
     const char *name;
     Class clazz;
@@ -150,9 +146,9 @@ typedef struct GPBExtensionDescription {
   union {
     const char *messageOrGroupClassName;
     union {
-     const char *name;
-     Class clazz;
-   } messageOrGroupClass;
+      const char *name;
+      Class clazz;
+    } messageOrGroupClass;
   };
   GPBEnumDescriptorFunc enumDescriptorFunc;
   int32_t fieldNumber;
@@ -161,20 +157,26 @@ typedef struct GPBExtensionDescription {
 } GPBExtensionDescription;
 
 typedef NS_OPTIONS(uint32_t, GPBDescriptorInitializationFlags) {
-  GPBDescriptorInitializationFlag_None              = 0,
+  GPBDescriptorInitializationFlag_None = 0,
   GPBDescriptorInitializationFlag_FieldsWithDefault = 1 << 0,
-  GPBDescriptorInitializationFlag_WireFormat        = 1 << 1,
+  GPBDescriptorInitializationFlag_WireFormat = 1 << 1,
 
   // This is used as a stopgap as we move from using class names to class
   // references. The runtime needs to support both until we allow a
   // breaking change in the runtime.
-  GPBDescriptorInitializationFlag_UsesClassRefs     = 1 << 2,
+  GPBDescriptorInitializationFlag_UsesClassRefs = 1 << 2,
 
   // This flag is used to indicate that the generated sources already contain
   // the `GPBFieldClearHasIvarOnZero` flag and it doesn't have to be computed
   // at startup. This allows older generated code to still work with the
   // current runtime library.
   GPBDescriptorInitializationFlag_Proto3OptionalKnown = 1 << 3,
+
+  // This flag is used to indicate that the generated sources already contain
+  // the `GPBFieldCloseEnum` flag and it doesn't have to be computed at startup.
+  // This allows the older generated code to still work with the current runtime
+  // library.
+  GPBDescriptorInitializationFlag_ClosedEnumSupportKnown = 1 << 4,
 };
 
 @interface GPBDescriptor () {
@@ -184,21 +186,14 @@ typedef NS_OPTIONS(uint32_t, GPBDescriptorInitializationFlags) {
   uint32_t storageSize_;
 }
 
-// fieldDescriptions have to be long lived, they are held as raw pointers.
-+ (instancetype)
-    allocDescriptorForClass:(Class)messageClass
-                  rootClass:(Class)rootClass
-                       file:(GPBFileDescriptor *)file
-                     fields:(void *)fieldDescriptions
-                 fieldCount:(uint32_t)fieldCount
-                storageSize:(uint32_t)storageSize
-                      flags:(GPBDescriptorInitializationFlags)flags;
-
-- (instancetype)initWithClass:(Class)messageClass
-                         file:(GPBFileDescriptor *)file
-                       fields:(NSArray *)fields
-                  storageSize:(uint32_t)storage
-                   wireFormat:(BOOL)wireFormat;
+// fieldDescriptions and fileDescription have to be long lived, they are held as raw pointers.
++ (instancetype)allocDescriptorForClass:(Class)messageClass
+                            messageName:(NSString *)messageName
+                        fileDescription:(GPBFileDescription *)fileDescription
+                                 fields:(void *)fieldDescriptions
+                             fieldCount:(uint32_t)fieldCount
+                            storageSize:(uint32_t)storageSize
+                                  flags:(GPBDescriptorInitializationFlags)flags;
 
 // Called right after init to provide extra information to avoid init having
 // an explosion of args. These pointers are recorded, so they are expected
@@ -209,10 +204,23 @@ typedef NS_OPTIONS(uint32_t, GPBDescriptorInitializationFlags) {
 - (void)setupExtraTextInfo:(const char *)extraTextFormatInfo;
 - (void)setupExtensionRanges:(const GPBExtensionRange *)ranges count:(int32_t)count;
 - (void)setupContainingMessageClass:(Class)msgClass;
-- (void)setupMessageClassNameSuffix:(NSString *)suffix;
 
-// Deprecated. Use setupContainingMessageClass instead.
+// Deprecated, these remain to support older versions of source generation.
++ (instancetype)allocDescriptorForClass:(Class)messageClass
+                                   file:(GPBFileDescriptor *)file
+                                 fields:(void *)fieldDescriptions
+                             fieldCount:(uint32_t)fieldCount
+                            storageSize:(uint32_t)storageSize
+                                  flags:(GPBDescriptorInitializationFlags)flags;
++ (instancetype)allocDescriptorForClass:(Class)messageClass
+                              rootClass:(Class)rootClass
+                                   file:(GPBFileDescriptor *)file
+                                 fields:(void *)fieldDescriptions
+                             fieldCount:(uint32_t)fieldCount
+                            storageSize:(uint32_t)storageSize
+                                  flags:(GPBDescriptorInitializationFlags)flags;
 - (void)setupContainingMessageClassName:(const char *)msgClassName;
+- (void)setupMessageClassNameSuffix:(NSString *)suffix;
 
 @end
 
@@ -220,8 +228,7 @@ typedef NS_OPTIONS(uint32_t, GPBDescriptorInitializationFlags) {
 - (instancetype)initWithPackage:(NSString *)package
                      objcPrefix:(NSString *)objcPrefix
                          syntax:(GPBFileSyntax)syntax;
-- (instancetype)initWithPackage:(NSString *)package
-                         syntax:(GPBFileSyntax)syntax;
+- (instancetype)initWithPackage:(NSString *)package syntax:(GPBFileSyntax)syntax;
 @end
 
 @interface GPBOneofDescriptor () {
@@ -244,39 +251,46 @@ typedef NS_OPTIONS(uint32_t, GPBDescriptorInitializationFlags) {
   SEL hasOrCountSel_;  // *Count for map<>/repeated fields, has* otherwise.
   SEL setHasSel_;
 }
-
-// Single initializer
-// description has to be long lived, it is held as a raw pointer.
-- (instancetype)initWithFieldDescription:(void *)description
-                         includesDefault:(BOOL)includesDefault
-                           usesClassRefs:(BOOL)usesClassRefs
-                     proto3OptionalKnown:(BOOL)proto3OptionalKnown
-                                  syntax:(GPBFileSyntax)syntax;
-
 @end
 
+typedef NS_OPTIONS(uint32_t, GPBEnumDescriptorInitializationFlags) {
+  GPBEnumDescriptorInitializationFlag_None = 0,
+
+  // Available: 1 << 0
+
+  // Marks this enum as a closed enum.
+  GPBEnumDescriptorInitializationFlag_IsClosed = 1 << 1,
+};
+
 @interface GPBEnumDescriptor ()
 // valueNames, values and extraTextFormatInfo have to be long lived, they are
 // held as raw pointers.
-+ (instancetype)
-    allocDescriptorForName:(NSString *)name
-                valueNames:(const char *)valueNames
-                    values:(const int32_t *)values
-                     count:(uint32_t)valueCount
-              enumVerifier:(GPBEnumValidationFunc)enumVerifier;
-+ (instancetype)
-    allocDescriptorForName:(NSString *)name
-                valueNames:(const char *)valueNames
-                    values:(const int32_t *)values
-                     count:(uint32_t)valueCount
-              enumVerifier:(GPBEnumValidationFunc)enumVerifier
-       extraTextFormatInfo:(const char *)extraTextFormatInfo;
++ (instancetype)allocDescriptorForName:(NSString *)name
+                            valueNames:(const char *)valueNames
+                                values:(const int32_t *)values
+                                 count:(uint32_t)valueCount
+                          enumVerifier:(GPBEnumValidationFunc)enumVerifier
+                                 flags:(GPBEnumDescriptorInitializationFlags)flags;
++ (instancetype)allocDescriptorForName:(NSString *)name
+                            valueNames:(const char *)valueNames
+                                values:(const int32_t *)values
+                                 count:(uint32_t)valueCount
+                          enumVerifier:(GPBEnumValidationFunc)enumVerifier
+                                 flags:(GPBEnumDescriptorInitializationFlags)flags
+                   extraTextFormatInfo:(const char *)extraTextFormatInfo;
 
-- (instancetype)initWithName:(NSString *)name
-                  valueNames:(const char *)valueNames
-                      values:(const int32_t *)values
-                       count:(uint32_t)valueCount
-                enumVerifier:(GPBEnumValidationFunc)enumVerifier;
+// Deprecated, these remain to support older versions of source generation.
++ (instancetype)allocDescriptorForName:(NSString *)name
+                            valueNames:(const char *)valueNames
+                                values:(const int32_t *)values
+                                 count:(uint32_t)valueCount
+                          enumVerifier:(GPBEnumValidationFunc)enumVerifier;
++ (instancetype)allocDescriptorForName:(NSString *)name
+                            valueNames:(const char *)valueNames
+                                values:(const int32_t *)values
+                                 count:(uint32_t)valueCount
+                          enumVerifier:(GPBEnumValidationFunc)enumVerifier
+                   extraTextFormatInfo:(const char *)extraTextFormatInfo;
 @end
 
 @interface GPBExtensionDescriptor () {
@@ -309,8 +323,7 @@ CF_EXTERN_C_BEGIN
 #pragma clang diagnostic ignored "-Wdirect-ivar-access"
 
 GPB_INLINE BOOL GPBFieldIsMapOrArray(GPBFieldDescriptor *field) {
-  return (field->description_->flags &
-          (GPBFieldRepeated | GPBFieldMapKeyMask)) != 0;
+  return (field->description_->flags & (GPBFieldRepeated | GPBFieldMapKeyMask)) != 0;
 }
 
 GPB_INLINE GPBDataType GPBGetFieldDataType(GPBFieldDescriptor *field) {
@@ -325,6 +338,10 @@ GPB_INLINE uint32_t GPBFieldNumber(GPBFieldDescriptor *field) {
   return field->description_->number;
 }
 
+GPB_INLINE BOOL GPBFieldIsClosedEnum(GPBFieldDescriptor *field) {
+  return (field->description_->flags & GPBFieldClosedEnum) != 0;
+}
+
 #pragma clang diagnostic pop
 
 uint32_t GPBFieldTag(GPBFieldDescriptor *self);
@@ -335,10 +352,6 @@ uint32_t GPBFieldTag(GPBFieldDescriptor *self);
 // would be the wire type for packed.
 uint32_t GPBFieldAlternateTag(GPBFieldDescriptor *self);
 
-GPB_INLINE BOOL GPBHasPreservingUnknownEnumSemantics(GPBFileSyntax syntax) {
-  return syntax == GPBFileSyntaxProto3;
-}
-
 GPB_INLINE BOOL GPBExtensionIsRepeated(GPBExtensionDescription *description) {
   return (description->options & GPBExtensionRepeated) != 0;
 }
@@ -353,22 +366,13 @@ GPB_INLINE BOOL GPBExtensionIsWireFormat(GPBExtensionDescription *description) {
 
 // Helper for compile time assets.
 #ifndef GPBInternalCompileAssert
-  #if __has_feature(c_static_assert) || __has_extension(c_static_assert)
-    #define GPBInternalCompileAssert(test, msg) _Static_assert((test), #msg)
-  #else
-    // Pre-Xcode 7 support.
-    #define GPBInternalCompileAssertSymbolInner(line, msg) GPBInternalCompileAssert ## line ## __ ## msg
-    #define GPBInternalCompileAssertSymbol(line, msg) GPBInternalCompileAssertSymbolInner(line, msg)
-    #define GPBInternalCompileAssert(test, msg) \
-        typedef char GPBInternalCompileAssertSymbol(__LINE__, msg) [ ((test) ? 1 : -1) ]
-  #endif  // __has_feature(c_static_assert) || __has_extension(c_static_assert)
-#endif // GPBInternalCompileAssert
+#define GPBInternalCompileAssert(test, msg) _Static_assert((test), #msg)
+#endif  // GPBInternalCompileAssert
 
 // Sanity check that there isn't padding between the field description
 // structures with and without a default.
 GPBInternalCompileAssert(sizeof(GPBMessageFieldDescriptionWithDefault) ==
-                         (sizeof(GPBGenericValue) +
-                          sizeof(GPBMessageFieldDescription)),
+                             (sizeof(GPBGenericValue) + sizeof(GPBMessageFieldDescription)),
                          DescriptionsWithDefault_different_size_than_expected);
 
 CF_EXTERN_C_END
