@@ -8,7 +8,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
   def test_acts_like_enumerator
     m = TestMessage.new
     (Enumerable.instance_methods - TestMessage.new.repeated_string.methods).each do |method_name|
-      assert m.repeated_string.respond_to?(method_name) == true, "does not respond to #{method_name}"
+      assert_respond_to m.repeated_string, method_name
     end
   end
 
@@ -20,9 +20,12 @@ class RepeatedFieldTest < Test::Unit::TestCase
       :iter_for_each_with_index, :dimensions, :copy_data, :copy_data_simple,
       :nitems, :iter_for_reverse_each, :indexes, :append, :prepend]
     arr_methods -= [:union, :difference, :filter!]
-    arr_methods -= [:intersection, :deconstruct] # ruby 2.7 methods we can ignore
+    # ruby 2.7 methods we can ignore
+    arr_methods -= [:intersection, :deconstruct, :resolve_feature_path]
+    # ruby 3.1 methods we can ignore
+    arr_methods -= [:intersect?]
     arr_methods.each do |method_name|
-      assert m.repeated_string.respond_to?(method_name) == true, "does not respond to #{method_name}"
+      assert_respond_to m.repeated_string, method_name
     end
   end
 
@@ -36,7 +39,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
     assert_equal -1_000_000, m.repeated_int64.first
     assert_equal 10, m.repeated_uint32.first
     assert_equal 1_000_000, m.repeated_uint64.first
-    assert_equal true, m.repeated_bool.first
+    assert m.repeated_bool.first
     assert_equal -1.01,  m.repeated_float.first.round(2)
     assert_equal -1.0000000000001, m.repeated_double.first
     assert_equal 'foo', m.repeated_string.first
@@ -56,7 +59,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
     assert_equal -1_000_001, m.repeated_int64.last
     assert_equal 11, m.repeated_uint32.last
     assert_equal 1_000_001, m.repeated_uint64.last
-    assert_equal false, m.repeated_bool.last
+    refute m.repeated_bool.last
     assert_equal -1.02, m.repeated_float.last.round(2)
     assert_equal -1.0000000000002, m.repeated_double.last
     assert_equal 'bar', m.repeated_string.last
@@ -81,8 +84,8 @@ class RepeatedFieldTest < Test::Unit::TestCase
     assert_equal 10, m.repeated_uint32.pop
     assert_equal 1_000_001, m.repeated_uint64.pop
     assert_equal 1_000_000, m.repeated_uint64.pop
-    assert_equal false, m.repeated_bool.pop
-    assert_equal true, m.repeated_bool.pop
+    refute m.repeated_bool.pop
+    assert m.repeated_bool.pop
     assert_equal -1.02,  m.repeated_float.pop.round(2)
     assert_equal -1.01,  m.repeated_float.pop.round(2)
     assert_equal -1.0000000000002, m.repeated_double.pop
@@ -121,11 +124,11 @@ class RepeatedFieldTest < Test::Unit::TestCase
 
   def test_empty?
     m = TestMessage.new
-    assert_equal true, m.repeated_string.empty?
+    assert_empty m.repeated_string
     m.repeated_string << 'foo'
-    assert_equal false, m.repeated_string.empty?
+    refute_empty m.repeated_string
     m.repeated_string << 'bar'
-    assert_equal false, m.repeated_string.empty?
+    refute_empty m.repeated_string
   end
 
   def test_array_accessor
@@ -251,7 +254,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
     m.repeated_string += reference_arr.clone
     assert_equal reference_arr, m.repeated_string
     reference_arr << 'fizz'
-    assert_not_equal reference_arr, m.repeated_string
+    refute_equal reference_arr, m.repeated_string
     m.repeated_string << 'fizz'
     assert_equal reference_arr, m.repeated_string
   end
@@ -265,7 +268,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
     hash = m.repeated_string.hash
     assert_equal hash, m.repeated_string.hash
     m.repeated_string << 'j'
-    assert_not_equal hash, m.repeated_string.hash
+    refute_equal hash, m.repeated_string.hash
   end
 
   def test_plus
@@ -326,18 +329,6 @@ class RepeatedFieldTest < Test::Unit::TestCase
     end
   end
 
-  def test_compact!
-    m = TestMessage.new
-    m.repeated_msg << TestMessage2.new(:foo => 1)
-    m.repeated_msg << nil
-    m.repeated_msg << TestMessage2.new(:foo => 2)
-    reference_arr = m.repeated_string.to_a
-
-    check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
-      arr.compact!
-    end
-  end
-
   def test_delete
     m = TestMessage.new
     reference_arr = %w(foo bar baz)
@@ -480,7 +471,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
     result = m.repeated_string.shuffle!
     assert_equal m.repeated_string, result
     # NOTE: sometimes it doesn't change the order...
-    # assert_not_equal m.repeated_string.to_a, orig_repeated_string.to_a
+    # refute_equal m.repeated_string.to_a, orig_repeated_string.to_a
   end
 
   def test_slice!
