@@ -1,41 +1,20 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
+#include "google/protobuf/reflection_tester.h"
 
-#include <google/protobuf/reflection_tester.h>
+#include <array>
 
 #include <gtest/gtest.h>
-#include <google/protobuf/map_field.h>
-#include <google/protobuf/message.h>
+#include "absl/container/flat_hash_map.h"
+#include "google/protobuf/map_field.h"
+#include "google/protobuf/message.h"
 
 // Must include last.
-#include <google/protobuf/port_def.inc>
+#include "google/protobuf/port_def.inc"
 
 namespace google {
 namespace protobuf {
@@ -45,79 +24,83 @@ MapReflectionTester::MapReflectionTester(const Descriptor* base_descriptor)
   const DescriptorPool* pool = base_descriptor->file()->pool();
   std::string package = base_descriptor->file()->package();
 
-  map_enum_foo_ = pool->FindEnumValueByName(package + ".MAP_ENUM_FOO");
-  map_enum_bar_ = pool->FindEnumValueByName(package + ".MAP_ENUM_BAR");
-  map_enum_baz_ = pool->FindEnumValueByName(package + ".MAP_ENUM_BAZ");
+  map_enum_foo_ =
+      pool->FindEnumValueByName(absl::StrCat(package, ".MAP_ENUM_FOO"));
+  map_enum_bar_ =
+      pool->FindEnumValueByName(absl::StrCat(package, ".MAP_ENUM_BAR"));
+  map_enum_baz_ =
+      pool->FindEnumValueByName(absl::StrCat(package, ".MAP_ENUM_BAZ"));
 
-  foreign_c_ = pool->FindFieldByName(package + ".ForeignMessage.c");
-  map_int32_int32_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapInt32Int32Entry.key");
-  map_int32_int32_val_ =
-      pool->FindFieldByName(package + ".TestMap.MapInt32Int32Entry.value");
-  map_int64_int64_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapInt64Int64Entry.key");
-  map_int64_int64_val_ =
-      pool->FindFieldByName(package + ".TestMap.MapInt64Int64Entry.value");
-  map_uint32_uint32_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapUint32Uint32Entry.key");
-  map_uint32_uint32_val_ =
-      pool->FindFieldByName(package + ".TestMap.MapUint32Uint32Entry.value");
-  map_uint64_uint64_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapUint64Uint64Entry.key");
-  map_uint64_uint64_val_ =
-      pool->FindFieldByName(package + ".TestMap.MapUint64Uint64Entry.value");
-  map_sint32_sint32_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapSint32Sint32Entry.key");
-  map_sint32_sint32_val_ =
-      pool->FindFieldByName(package + ".TestMap.MapSint32Sint32Entry.value");
-  map_sint64_sint64_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapSint64Sint64Entry.key");
-  map_sint64_sint64_val_ =
-      pool->FindFieldByName(package + ".TestMap.MapSint64Sint64Entry.value");
-  map_fixed32_fixed32_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapFixed32Fixed32Entry.key");
-  map_fixed32_fixed32_val_ =
-      pool->FindFieldByName(package + ".TestMap.MapFixed32Fixed32Entry.value");
-  map_fixed64_fixed64_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapFixed64Fixed64Entry.key");
-  map_fixed64_fixed64_val_ =
-      pool->FindFieldByName(package + ".TestMap.MapFixed64Fixed64Entry.value");
-  map_sfixed32_sfixed32_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapSfixed32Sfixed32Entry.key");
+  foreign_c_ =
+      pool->FindFieldByName(absl::StrCat(package, ".ForeignMessage.c"));
+  map_int32_int32_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapInt32Int32Entry.key"));
+  map_int32_int32_val_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapInt32Int32Entry.value"));
+  map_int64_int64_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapInt64Int64Entry.key"));
+  map_int64_int64_val_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapInt64Int64Entry.value"));
+  map_uint32_uint32_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapUint32Uint32Entry.key"));
+  map_uint32_uint32_val_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapUint32Uint32Entry.value"));
+  map_uint64_uint64_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapUint64Uint64Entry.key"));
+  map_uint64_uint64_val_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapUint64Uint64Entry.value"));
+  map_sint32_sint32_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapSint32Sint32Entry.key"));
+  map_sint32_sint32_val_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapSint32Sint32Entry.value"));
+  map_sint64_sint64_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapSint64Sint64Entry.key"));
+  map_sint64_sint64_val_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapSint64Sint64Entry.value"));
+  map_fixed32_fixed32_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapFixed32Fixed32Entry.key"));
+  map_fixed32_fixed32_val_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapFixed32Fixed32Entry.value"));
+  map_fixed64_fixed64_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapFixed64Fixed64Entry.key"));
+  map_fixed64_fixed64_val_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapFixed64Fixed64Entry.value"));
+  map_sfixed32_sfixed32_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapSfixed32Sfixed32Entry.key"));
   map_sfixed32_sfixed32_val_ = pool->FindFieldByName(
-      package + ".TestMap.MapSfixed32Sfixed32Entry.value");
-  map_sfixed64_sfixed64_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapSfixed64Sfixed64Entry.key");
+      absl::StrCat(package, ".TestMap.MapSfixed32Sfixed32Entry.value"));
+  map_sfixed64_sfixed64_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapSfixed64Sfixed64Entry.key"));
   map_sfixed64_sfixed64_val_ = pool->FindFieldByName(
-      package + ".TestMap.MapSfixed64Sfixed64Entry.value");
-  map_int32_float_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapInt32FloatEntry.key");
-  map_int32_float_val_ =
-      pool->FindFieldByName(package + ".TestMap.MapInt32FloatEntry.value");
-  map_int32_double_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapInt32DoubleEntry.key");
-  map_int32_double_val_ =
-      pool->FindFieldByName(package + ".TestMap.MapInt32DoubleEntry.value");
-  map_bool_bool_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapBoolBoolEntry.key");
-  map_bool_bool_val_ =
-      pool->FindFieldByName(package + ".TestMap.MapBoolBoolEntry.value");
-  map_string_string_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapStringStringEntry.key");
-  map_string_string_val_ =
-      pool->FindFieldByName(package + ".TestMap.MapStringStringEntry.value");
-  map_int32_bytes_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapInt32BytesEntry.key");
-  map_int32_bytes_val_ =
-      pool->FindFieldByName(package + ".TestMap.MapInt32BytesEntry.value");
-  map_int32_enum_key_ =
-      pool->FindFieldByName(package + ".TestMap.MapInt32EnumEntry.key");
-  map_int32_enum_val_ =
-      pool->FindFieldByName(package + ".TestMap.MapInt32EnumEntry.value");
+      absl::StrCat(package, ".TestMap.MapSfixed64Sfixed64Entry.value"));
+  map_int32_float_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapInt32FloatEntry.key"));
+  map_int32_float_val_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapInt32FloatEntry.value"));
+  map_int32_double_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapInt32DoubleEntry.key"));
+  map_int32_double_val_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapInt32DoubleEntry.value"));
+  map_bool_bool_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapBoolBoolEntry.key"));
+  map_bool_bool_val_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapBoolBoolEntry.value"));
+  map_string_string_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapStringStringEntry.key"));
+  map_string_string_val_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapStringStringEntry.value"));
+  map_int32_bytes_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapInt32BytesEntry.key"));
+  map_int32_bytes_val_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapInt32BytesEntry.value"));
+  map_int32_enum_key_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapInt32EnumEntry.key"));
+  map_int32_enum_val_ = pool->FindFieldByName(
+      absl::StrCat(package, ".TestMap.MapInt32EnumEntry.value"));
   map_int32_foreign_message_key_ = pool->FindFieldByName(
-      package + ".TestMap.MapInt32ForeignMessageEntry.key");
+      absl::StrCat(package, ".TestMap.MapInt32ForeignMessageEntry.key"));
   map_int32_foreign_message_val_ = pool->FindFieldByName(
-      package + ".TestMap.MapInt32ForeignMessageEntry.value");
+      absl::StrCat(package, ".TestMap.MapInt32ForeignMessageEntry.value"));
 
   EXPECT_FALSE(map_enum_foo_ == nullptr);
   EXPECT_FALSE(map_enum_bar_ == nullptr);
@@ -193,7 +176,7 @@ MapReflectionTester::MapReflectionTester(const Descriptor* base_descriptor)
       map_int32_foreign_message_key_,
       map_int32_foreign_message_val_};
   for (const FieldDescriptor* fdesc : all_map_descriptors) {
-    GOOGLE_CHECK(fdesc->containing_type() != nullptr) << fdesc->name();
+    ABSL_CHECK(fdesc->containing_type() != nullptr) << fdesc->name();
     if (fdesc->name() == "key") {
       EXPECT_EQ(fdesc->containing_type()->map_key(), fdesc);
     } else {
@@ -201,13 +184,17 @@ MapReflectionTester::MapReflectionTester(const Descriptor* base_descriptor)
       EXPECT_EQ(fdesc->containing_type()->map_value(), fdesc);
     }
   }
+
+  // Must be heap allocated.
+  EXPECT_NE(long_string().capacity(), std::string().capacity());
+  EXPECT_NE(long_string_2().capacity(), std::string().capacity());
 }
 
 // Shorthand to get a FieldDescriptor for a field of unittest::TestMap.
 const FieldDescriptor* MapReflectionTester::F(const std::string& name) {
   const FieldDescriptor* result = nullptr;
   result = base_descriptor_->FindFieldByName(name);
-  GOOGLE_CHECK(result != nullptr);
+  ABSL_CHECK(result != nullptr);
   return result;
 }
 
@@ -289,14 +276,14 @@ void MapReflectionTester::SetMapFieldsViaReflection(Message* message) {
 
   sub_message = reflection->AddMessage(message, F("map_string_string"));
   sub_message->GetReflection()->SetString(sub_message, map_string_string_key_,
-                                          "0");
+                                          long_string());
   sub_message->GetReflection()->SetString(sub_message, map_string_string_val_,
-                                          "0");
+                                          long_string());
 
   sub_message = reflection->AddMessage(message, F("map_int32_bytes"));
   sub_message->GetReflection()->SetInt32(sub_message, map_int32_bytes_key_, 0);
   sub_message->GetReflection()->SetString(sub_message, map_int32_bytes_val_,
-                                          "0");
+                                          long_string());
 
   sub_message = reflection->AddMessage(message, F("map_int32_enum"));
   sub_message->GetReflection()->SetInt32(sub_message, map_int32_enum_key_, 0);
@@ -384,14 +371,14 @@ void MapReflectionTester::SetMapFieldsViaReflection(Message* message) {
 
   sub_message = reflection->AddMessage(message, F("map_string_string"));
   sub_message->GetReflection()->SetString(sub_message, map_string_string_key_,
-                                          "1");
+                                          long_string_2());
   sub_message->GetReflection()->SetString(sub_message, map_string_string_val_,
-                                          "1");
+                                          long_string_2());
 
   sub_message = reflection->AddMessage(message, F("map_int32_bytes"));
   sub_message->GetReflection()->SetInt32(sub_message, map_int32_bytes_key_, 1);
   sub_message->GetReflection()->SetString(sub_message, map_int32_bytes_val_,
-                                          "1");
+                                          long_string_2());
 
   sub_message = reflection->AddMessage(message, F("map_int32_enum"));
   sub_message->GetReflection()->SetInt32(sub_message, map_int32_enum_key_, 1);
@@ -489,19 +476,19 @@ void MapReflectionTester::SetMapFieldsViaMapReflection(Message* message) {
                                                  map_key, &map_val));
   map_val.SetBoolValue(false);
 
-  map_key.SetStringValue("0");
+  map_key.SetStringValue(long_string());
   EXPECT_FALSE(reflection->LookupMapValue(*message, F("map_string_string"),
                                           map_key, &map_val_const));
   EXPECT_TRUE(reflection->InsertOrLookupMapValue(
       message, F("map_string_string"), map_key, &map_val));
-  map_val.SetStringValue("0");
+  map_val.SetStringValue(long_string());
 
   map_key.SetInt32Value(0);
   EXPECT_FALSE(reflection->LookupMapValue(*message, F("map_int32_bytes"),
                                           map_key, &map_val_const));
   EXPECT_TRUE(reflection->InsertOrLookupMapValue(message, F("map_int32_bytes"),
                                                  map_key, &map_val));
-  map_val.SetStringValue("0");
+  map_val.SetStringValue(long_string());
 
   map_key.SetInt32Value(0);
   EXPECT_FALSE(reflection->LookupMapValue(*message, F("map_int32_enum"),
@@ -589,15 +576,15 @@ void MapReflectionTester::SetMapFieldsViaMapReflection(Message* message) {
                                      &map_val);
   map_val.SetBoolValue(true);
 
-  map_key.SetStringValue("1");
+  map_key.SetStringValue(long_string_2());
   reflection->InsertOrLookupMapValue(message, F("map_string_string"), map_key,
                                      &map_val);
-  map_val.SetStringValue("1");
+  map_val.SetStringValue(long_string_2());
 
   map_key.SetInt32Value(1);
   reflection->InsertOrLookupMapValue(message, F("map_int32_bytes"), map_key,
                                      &map_val);
-  map_val.SetStringValue("1");
+  map_val.SetStringValue(long_string_2());
 
   map_key.SetInt32Value(1);
   reflection->InsertOrLookupMapValue(message, F("map_int32_enum"), map_key,
@@ -738,7 +725,7 @@ void MapReflectionTester::ModifyMapFieldsViaReflection(Message* message) {
                                      &map_val);
   map_val.SetBoolValue(false);
 
-  map_key.SetStringValue("1");
+  map_key.SetStringValue(long_string_2());
   reflection->InsertOrLookupMapValue(message, F("map_string_string"), map_key,
                                      &map_val);
   map_val.SetStringValue("2");
@@ -888,7 +875,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
   ASSERT_EQ(2, reflection->FieldSize(message, F("map_int32_foreign_message")));
 
   {
-    std::map<int32_t, int32_t> map;
+    absl::flat_hash_map<int32_t, int32_t> map;
     map[0] = 0;
     map[1] = 1;
     for (int i = 0; i < 2; i++) {
@@ -916,7 +903,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<int64_t, int64_t> map;
+    absl::flat_hash_map<int64_t, int64_t> map;
     map[0] = 0;
     map[1] = 1;
     for (int i = 0; i < 2; i++) {
@@ -944,7 +931,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<uint32_t, uint32_t> map;
+    absl::flat_hash_map<uint32_t, uint32_t> map;
     map[0] = 0;
     map[1] = 1;
     for (int i = 0; i < 2; i++) {
@@ -972,7 +959,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<uint64_t, uint64_t> map;
+    absl::flat_hash_map<uint64_t, uint64_t> map;
     map[0] = 0;
     map[1] = 1;
     for (int i = 0; i < 2; i++) {
@@ -1000,7 +987,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<int32_t, int32_t> map;
+    absl::flat_hash_map<int32_t, int32_t> map;
     map[0] = 0;
     map[1] = 1;
     for (int i = 0; i < 2; i++) {
@@ -1028,7 +1015,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<int64_t, int64_t> map;
+    absl::flat_hash_map<int64_t, int64_t> map;
     map[0] = 0;
     map[1] = 1;
     for (int i = 0; i < 2; i++) {
@@ -1056,7 +1043,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<uint32_t, uint32_t> map;
+    absl::flat_hash_map<uint32_t, uint32_t> map;
     map[0] = 0;
     map[1] = 1;
     for (int i = 0; i < 2; i++) {
@@ -1084,7 +1071,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<uint64_t, uint64_t> map;
+    absl::flat_hash_map<uint64_t, uint64_t> map;
     map[0] = 0;
     map[1] = 1;
     for (int i = 0; i < 2; i++) {
@@ -1112,7 +1099,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<int32_t, int32_t> map;
+    absl::flat_hash_map<int32_t, int32_t> map;
     map[0] = 0;
     map[1] = 1;
     for (int i = 0; i < 2; i++) {
@@ -1141,7 +1128,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<int64_t, int64_t> map;
+    absl::flat_hash_map<int64_t, int64_t> map;
     map[0] = 0;
     map[1] = 1;
     for (int i = 0; i < 2; i++) {
@@ -1170,7 +1157,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<int32_t, float> map;
+    absl::flat_hash_map<int32_t, float> map;
     map[0] = 0.0;
     map[1] = 1.0;
     for (int i = 0; i < 2; i++) {
@@ -1198,7 +1185,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<int32_t, double> map;
+    absl::flat_hash_map<int32_t, double> map;
     map[0] = 0.0;
     map[1] = 1.0;
     for (int i = 0; i < 2; i++) {
@@ -1226,9 +1213,9 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<bool, bool> map;
-    map[false] = false;
-    map[true] = true;
+    std::array<bool, 2> map;
+    map[0] = false;
+    map[1] = true;
     std::vector<bool> keys = {false, true};
     std::vector<bool> vals = {false, true};
     for (int i = 0; i < 2; i++) {
@@ -1243,7 +1230,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
                                                          map_bool_bool_key_);
         bool val = sub_message->GetReflection()->GetBool(*sub_message,
                                                          map_bool_bool_val_);
-        EXPECT_EQ(map[key], val);
+        EXPECT_EQ(map[key ? 1 : 0], val);
       } else {
         // Check with Map Reflection
         map_key.SetBoolValue(keys[i]);
@@ -1256,11 +1243,11 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<std::string, std::string> map;
-    map["0"] = "0";
-    map["1"] = "1";
-    std::vector<std::string> keys = {"0", "1"};
-    std::vector<std::string> vals = {"0", "1"};
+    absl::flat_hash_map<std::string, std::string> map;
+    map[long_string()] = long_string();
+    map[long_string_2()] = long_string_2();
+    std::vector<std::string> keys = {long_string(), long_string_2()};
+    std::vector<std::string> vals = {long_string(), long_string_2()};
     for (int i = 0; i < 2; i++) {
       const internal::MapFieldBase& map_field =
           reflection->GetRaw<internal::MapFieldBase>(message,
@@ -1286,9 +1273,9 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<int32_t, std::string> map;
-    map[0] = "0";
-    map[1] = "1";
+    absl::flat_hash_map<int32_t, std::string> map;
+    map[0] = long_string();
+    map[1] = long_string_2();
     for (int i = 0; i < 2; i++) {
       const internal::MapFieldBase& map_field =
           reflection->GetRaw<internal::MapFieldBase>(message,
@@ -1314,7 +1301,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<int32_t, const EnumValueDescriptor*> map;
+    absl::flat_hash_map<int32_t, const EnumValueDescriptor*> map;
     map[0] = map_enum_bar_;
     map[1] = map_enum_baz_;
     for (int i = 0; i < 2; i++) {
@@ -1342,7 +1329,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    std::map<int32_t, int32_t> map;
+    absl::flat_hash_map<int32_t, int32_t> map;
     map[0] = 0;
     map[1] = 1;
     for (int i = 0; i < 2; i++) {
@@ -1403,7 +1390,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
   ASSERT_EQ(2, reflection->FieldSize(*message, F("map_int32_foreign_message")));
 
   {
-    std::map<int32_t, int32_t> map;
+    absl::flat_hash_map<int32_t, int32_t> map;
     map[0] = 0;
     map[1] = 1;
     int size = 0;
@@ -1422,7 +1409,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     EXPECT_EQ(size, 2);
   }
   {
-    std::map<int64_t, int64_t> map;
+    absl::flat_hash_map<int64_t, int64_t> map;
     map[0] = 0;
     map[1] = 1;
     for (MapIterator iter = reflection->MapBegin(message, F("map_int64_int64"));
@@ -1432,7 +1419,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     }
   }
   {
-    std::map<uint32_t, uint32_t> map;
+    absl::flat_hash_map<uint32_t, uint32_t> map;
     map[0] = 0;
     map[1] = 1;
     for (MapIterator iter =
@@ -1443,7 +1430,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     }
   }
   {
-    std::map<uint64_t, uint64_t> map;
+    absl::flat_hash_map<uint64_t, uint64_t> map;
     map[0] = 0;
     map[1] = 1;
     for (MapIterator iter =
@@ -1454,7 +1441,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     }
   }
   {
-    std::map<int32_t, int32_t> map;
+    absl::flat_hash_map<int32_t, int32_t> map;
     map[0] = 0;
     map[1] = 1;
     for (MapIterator iter =
@@ -1465,7 +1452,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     }
   }
   {
-    std::map<int64_t, int64_t> map;
+    absl::flat_hash_map<int64_t, int64_t> map;
     map[0] = 0;
     map[1] = 1;
     for (MapIterator iter =
@@ -1476,7 +1463,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     }
   }
   {
-    std::map<uint32_t, uint32_t> map;
+    absl::flat_hash_map<uint32_t, uint32_t> map;
     map[0] = 0;
     map[1] = 1;
     for (MapIterator iter =
@@ -1488,7 +1475,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     }
   }
   {
-    std::map<uint64_t, uint64_t> map;
+    absl::flat_hash_map<uint64_t, uint64_t> map;
     map[0] = 0;
     map[1] = 1;
     for (MapIterator iter =
@@ -1500,7 +1487,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     }
   }
   {
-    std::map<int32_t, int32_t> map;
+    absl::flat_hash_map<int32_t, int32_t> map;
     map[0] = 0;
     map[1] = 1;
     for (MapIterator iter =
@@ -1512,7 +1499,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     }
   }
   {
-    std::map<int32_t, float> map;
+    absl::flat_hash_map<int32_t, float> map;
     map[0] = 0.0;
     map[1] = 1.0;
     for (MapIterator iter = reflection->MapBegin(message, F("map_int32_float"));
@@ -1522,7 +1509,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     }
   }
   {
-    std::map<int32_t, double> map;
+    absl::flat_hash_map<int32_t, double> map;
     map[0] = 0.0;
     map[1] = 1.0;
     for (MapIterator iter =
@@ -1533,19 +1520,19 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     }
   }
   {
-    std::map<bool, bool> map;
-    map[false] = false;
-    map[true] = true;
+    std::array<bool, 2> map;
+    map[0] = false;
+    map[1] = true;
     for (MapIterator iter = reflection->MapBegin(message, F("map_bool_bool"));
          iter != reflection->MapEnd(message, F("map_bool_bool")); ++iter) {
-      EXPECT_EQ(map[iter.GetKey().GetBoolValue()],
+      EXPECT_EQ(map[iter.GetKey().GetBoolValue() ? 1 : 0],
                 iter.GetValueRef().GetBoolValue());
     }
   }
   {
-    std::map<std::string, std::string> map;
-    map["0"] = "0";
-    map["1"] = "1";
+    absl::flat_hash_map<std::string, std::string> map;
+    map[long_string()] = long_string();
+    map[long_string_2()] = long_string_2();
     int size = 0;
     for (MapIterator iter =
              reflection->MapBegin(message, F("map_string_string"));
@@ -1563,9 +1550,9 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     EXPECT_EQ(size, 2);
   }
   {
-    std::map<int32_t, std::string> map;
-    map[0] = "0";
-    map[1] = "1";
+    absl::flat_hash_map<int32_t, std::string> map;
+    map[0] = long_string();
+    map[1] = long_string_2();
     for (MapIterator iter = reflection->MapBegin(message, F("map_int32_bytes"));
          iter != reflection->MapEnd(message, F("map_int32_bytes")); ++iter) {
       EXPECT_EQ(map[iter.GetKey().GetInt32Value()],
@@ -1573,7 +1560,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     }
   }
   {
-    std::map<int32_t, const EnumValueDescriptor*> map;
+    absl::flat_hash_map<int32_t, const EnumValueDescriptor*> map;
     map[0] = map_enum_bar_;
     map[1] = map_enum_baz_;
     for (MapIterator iter = reflection->MapBegin(message, F("map_int32_enum"));
@@ -1583,7 +1570,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     }
   }
   {
-    std::map<int32_t, int32_t> map;
+    absl::flat_hash_map<int32_t, int32_t> map;
     map[0] = 0;
     map[1] = 1;
     int size = 0;
@@ -1670,4 +1657,4 @@ void MapReflectionTester::ExpectClearViaReflectionIterator(Message* message) {
 }  // namespace protobuf
 }  // namespace google
 
-#include <google/protobuf/port_undef.inc>
+#include "google/protobuf/port_undef.inc"
